(*
    Ocaml XML-RPC support.
    Copyright (C) 2004 Shawn Wagner <raevnos@pennmush.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*)


(** XML-RPC Client routines *)

(**
  {1 Remote method invocation}

   [new remote xmlrpc-server method] returns an object that can be
   used to execute the foreign method. Its [simple_call] method is
   convienience for a function that only takes one argument, and
   [zero_call] for functions without arguments. [call] must be used
   for all others.

   While there's nothing stopping people from using this class
   directly, using functions generated by oxridl is much nicer and
   simpler, because it handles the conversion btween XML-RPC types and
   ml types automatically, for the most part. Please do so if possible.
*)


(** {2 Exceptions} *)

(** This exception (Or {!XmlRPCTypes.Bad_type}) is raised if there's a
  technical problem with the processing -- can't connect to the server,
  the server returned something besides valid XML-RPC xml, or whatever.
*)
exception Request_failed of string

(** This exception is raised if the request completes successfully,
  but returned a &lt;fault&gt; instead of a &lt;params&gt; in the
  &lt;methodResponse&gt; node. In other words, error generated from
  the server because of the request, not in the transaction. It holds a
  {!XmlRPCTypes.xrs} with the fault information.
*)
exception Request_fault of XmlRPCTypes.fault

(** {2 Remote calls} *)

(** This is the string sent as the user-agent argument in HTTP headers. *)
val client_version: string

(** This class encapsulates a remote procedure. See above for details. *)
class remote: string -> string -> object
  method zero_call: unit -> XmlRPCTypes.t (** Remote function without arguments *)
  method simple_call: XmlRPCTypes.t -> XmlRPCTypes.t  (** Remote function with one argument *)
  method call: XmlRPCTypes.t list -> XmlRPCTypes.t (** Remote function with multiple arguments *)
end

