(*
    Ocaml XML-RPC support.
    Copyright (C) 2004 Shawn Wagner <raevnos@pennmush.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*)

(** XML-RPC Server routines *)

(** These functions provide ways to parse and execute XML-RPC requests. Receiving the requests is handled in other modules, to provide a variety of ways to get them (standalone daemon, CGI, mod_ocaml, etc. At the moment, only the first two are supported. 

Generally, you won't need to call these functions yourself. They're used in the source generated from idl files.

*)

(** {1 Types} *)

(** The type that describes a request handler *)
type t

(** The type of request sources. *)
class type source = object
  method get_request: unit -> string (** Returns the unparsed headers and body of a request *)
  method send_reply: string -> unit (** Send the XML in string back *)
end

exception Error of string * int

(** Create a handler *)
val make: source -> t

(** {1 Registering request handlers.} *)

(** Registers a callback function for a method with a handler. *)
val register_callback: t -> string -> (XmlRPCTypes.t array -> XmlRPCTypes.t) -> unit 

(** Unregisters a recognized method. *)
val remove_callback: t -> string -> unit

(** Is a method handled or not? *)
val is_handled: t -> string -> bool

(** Returns a list of all handled function requests *)
val list_functions: t -> string list

(** {1 Handling requests} *)

val server: string

(** Starts an infinite loop waiting for requests and dealing with them. *)
val run: t -> unit
